import os
import re
import sys
import subprocess
import gettext
import locale
from pathlib import Path

# Localization implementation
current_lang, _ = locale.getlocale()
if not current_lang:
    current_lang = 'en'

system_locale_dir = "/usr/share/locale"

def translation_exists(lang, base_dir):
    mo_file = os.path.join(base_dir, lang, "LC_MESSAGES", "krbpass.mo")
    return os.path.exists(mo_file)

if current_lang.startswith("en"):
    _ = lambda s: s
else:
    translation = gettext.translation("krbpass", localedir=system_locale_dir, languages=[current_lang], fallback=True)
    _ = translation.gettext

def get_realms(path="/etc/krb5.conf"):
    """Parse krb5.conf to extract available Kerberos realms and their KDCs.

    Args:
        path: Path to the krb5.conf file.

    Returns:
        Dictionary of realms with their associated KDCs or None if file is missing.
    """
    if not os.path.exists(path):
        return None

    realms = {}
    current_realm = None

    with open(path, "r") as file:
        for line in file:
            line = line.strip()
            if not line or line.startswith("#"):
                continue

            if "default_realm" in line and "=" in line:
                match = re.search(r'([A-Z0-9.-_]+)', line.split("=", 1)[-1].strip())
                default_realm = match.group(1) if match else None
                if default_realm:
                    realms.setdefault(default_realm, [])

            if line.endswith("{"):
                match = re.search(r'([A-Z0-9.-_]+)', line.split("{", 1)[0].strip())
                current_realm = match.group(1) if match else None
                if current_realm:
                    realms[current_realm] = []
                continue

            if current_realm and "kdc" in line and "=" in line:
                match = re.search(r'([A-Za-z0-9.-_]+)', line.split("=", 1)[-1].strip())
                kdc = match.group(1) if match else None
                if kdc:
                    realms[current_realm].append(kdc)

            if line == "}":
                current_realm = None

    return realms


def change_password(old_pass, new_pass, user_name, realm):
    """Change Kerberos password for a domain user.

    Args:
        old_pass: String with old user password
        new_pass: String with new user password
        user_name: String with user name
        realm: String with kerberos realm name

    Returns:
        Tuple (output, error, domain controller)
    """
    get_dc_cmd = f"adcli info {realm} | grep 'domain-controller =' | awk '{{print $3}}'"
    get_dc = subprocess.Popen(get_dc_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    dc, dc_err = get_dc.communicate()
    dc = dc.decode('utf-8').strip()

    if get_dc.returncode != 0:
        return None, dc_err.decode('utf-8'), None

    command = f"kpasswd {user_name}@{realm}"
    process = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    output, error = process.communicate(input=f"{old_pass}\n{new_pass}\n{new_pass}".encode())

    return output, error, dc

def get_username():
    """Get name of current user"""
    result = os.popen("whoami")
    result = ', '.join(result)
    result = result.replace("\n", "")
    return result
