import gi
gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')
from gi.repository import Gtk, Adw, GLib

from .utils import get_username, get_realms, change_password, _

@Gtk.Template(resource_path='/com/softline/krbpass/window.ui')
class KrbPassWindow(Adw.ApplicationWindow):
    __gtype_name__ = "KrbPassWindow"

    # Template widgets
    entry_user = Gtk.Template.Child()
    realm_box = Gtk.Template.Child()
    entry_old_pass = Gtk.Template.Child()
    entry_new_pass = Gtk.Template.Child()
    entry_confirm_pass = Gtk.Template.Child()
    change_button = Gtk.Template.Child()
    spinner = Gtk.Template.Child()
    toast_overlay = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.set_icon_name("com.softline.krbpass")


        # Set up the UI elements that can't be defined in the template
        self.entry_user.set_text(get_username())

        # Set up the realms dropdown
        realms_dict = get_realms()
        if not realms_dict:
            self.show_dialog(_("Password Change"), _("No available realms"),\
                True, True)
            self.change_button.set_sensitive(False)
            return

        # Set up model for realm dropdown
        self.realms_list = list(realms_dict.keys())
        string_list = Gtk.StringList.new(self.realms_list)
        self.realm_box.set_model(string_list)

        # Connect signals
        self.change_button.connect("clicked", self.on_change_button_clicked)

    def on_change_button_clicked(self, button):
        """Handle change password button click"""
        user_name = self.entry_user.get_text()
        old_pass = self.entry_old_pass.get_text()
        new_pass = self.entry_new_pass.get_text()
        new_pass_repeat = self.entry_confirm_pass.get_text()

        selected = self.realm_box.get_selected()
        if selected >= 0 and selected < len(self.realms_list):
            realm = self.realms_list[selected]
        else:
            self.show_toast(_("Please select a realm"))
            return

        # Validate inputs
        if not all([user_name, old_pass, new_pass, new_pass_repeat, realm]):
            self.show_toast(_("All fields must be filled!"))
            return

        if new_pass != new_pass_repeat:
            self.show_toast(_("Passwords do not match!"))
            return

        # Start the password change process
        self.start_password_change(user_name, old_pass, new_pass, realm)

    def show_toast(self, message, timeout=3):
        """Show a toast notification

        Args:
            message: Message to display
            timeout: Time in seconds to show the toast
        """
        toast = Adw.Toast.new(message)
        toast.set_timeout(timeout)
        self.toast_overlay.add_toast(toast)

    def start_password_change(self, user_name, old_pass, new_pass, realm):
        """Prepare UI for password change and start the operation

        Args:
            user_name: String with user name
            old_pass: String with old password
            new_pass: String with new password
            realm: String with realm name
        """
        self.change_button.set_sensitive(False)
        self.spinner.set_visible(True)
        self.spinner.start()

        GLib.idle_add(self._perform_password_change, user_name, old_pass,\
            new_pass, realm)

    def _perform_password_change(self, user_name, old_pass, new_pass, realm):
        """Execute password change operation in background

        Args:
            user_name: String with user name
            old_pass: String with old password
            new_pass: String with new password
            realm: String with realm name
        """
        out, err, dc = change_password(old_pass, new_pass, user_name, realm)

        GLib.idle_add(self._handle_password_change_result, out, err)
        return False

    def _handle_password_change_result(self, out, err):
        """Handle the result of the password change operation

        Args:
            out: Output from the password change command
            err: Error output from the password change command
        """
        self.spinner.stop()
        self.spinner.set_visible(False)
        self.change_button.set_sensitive(True)

        if not err and out is not None:
            if out.decode('utf-8').splitlines()[-1] == "Password changed.":
                self.show_dialog(_("Success"),\
                    _("Password successfully changed!"), False)
            else:
                message = "\n".join(out.decode('utf-8').splitlines()[3:])
                self.show_dialog(_("Password Change"), message, False)
            print("\n".join(out.decode('utf-8').splitlines()[3:]))
        else:
            if err:
                message = err.decode('utf-8').removeprefix("kpasswd: ")
                self.show_dialog(_("Password Change Error"), message, True)
                print(_("Password Change Error") + ": " +\
                    err.decode('utf-8').removeprefix("kpasswd: "))

        self.entry_old_pass.set_text("")
        self.entry_new_pass.set_text("")
        self.entry_confirm_pass.set_text("")

        return False

    def show_dialog(self, title, message, is_error, auto_close=False):
        """Show a dialog with the given message

        Args:
            title: Dialog title
            message: Dialog message
            is_error: Whether this is an error message
            auto_close: If toast should close the app whe toast is closed
        """
        dialog = Adw.MessageDialog.new(self, title, message)
        dialog.add_response("ok", _("OK"))
        dialog.set_default_response("ok")
        dialog.set_response_appearance("ok", Adw.ResponseAppearance.SUGGESTED)
        dialog.connect("response", lambda d, r: (d.destroy(), auto_close\
            and self.get_application().quit()))
        dialog.present()

